C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfgkvale (crec, rval, idbg, ierr)

C @(#)Subroutine: rjfgkvale          Author: R.A. Jansen -- Oct  6, 2009
C @(#)
C @(#)  Purpose:  Read  a real-valued keyword value  'rval'  from a FITS
C @(#)    header record 'crec',  already known to contain  a real-valued
C @(#)    keyword.  'rval' is declared as a variable of the default REAL
C @(#)    data type.  For most host OS and compiler implementations this
C @(#)    corresponds to a 32-bit (REAL*4)  floating point real.  If the
C @(#)    keyword value was written  in free format  (allowed in v3.0 of
C @(#)    the FITS standard),  'crec'  will be modified  and returned in
C @(#)    fixed format upon exit.  If verbosity level 'idbg'>0, additio-
C @(#)    nal messaged may be printed.  Error status 'ierr' will be non-
C @(#)    zero upon exit,  should any error condition  be detected.  Its
C @(#)    value (if >2000) uniquely identifies a known error condition.
C @(#)
C @(#)  Calling sequence: CALL rjfgkvale (crec, rval, idbg, ierr), where
C @(#)    'crec'  is a  character string  with a  declared length  of 80
C @(#)    characters and  'rval'  is a variable of the default REAL data
C @(#)    type.  'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note1: this task is meant to be run after 'rjfvkval'  (if 'ityp'
C @(#)    has a return value of 6) and 'rjfgkwrd'.
C @(#)  Note2: 32-bit floating point REALs generally represent at most 6
C @(#)    or 7 significant digits faithfully.  If more precision appears
C @(#)    necessary, this task will warn but exit with 0 error status.
C @(#)
C @(#)  Dependencies:	clen, cinit, cidx, lenc, ucase
C @(#)
C @(#)  Revision history:
C @(#)    Oct  5, 2009 [RAJ] -- Initial version of this routine.
C @(#)    Oct  6, 2009 [RAJ] -- Added check for number of digits in man-
C @(#)                          tissa and any exponent. Improved evalua-
C @(#)                          tion of required precision and error.
C @(#)    Oct 13, 2009 [RAJ] -- Fixed bug (incorrect 'E'->'D' in 'ctmp',
C @(#)                          instead of in 'cval').
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80
      REAL	rval
      INTEGER	idbg, ierr

C --- Declaration of local variables...
      CHARACTER ctmp*80, cval*80, dbid*19, cc
      REAL*8	rtmp, rdev
      INTEGER	clen, cidx, lenc, i, j, k, m, ic, nd, np

C     The following variables are shorthand integer equivalents in the
C     ASCII collating sequence of space, equals sign, digits 0 and 9, E,
C     D, plus and minus, decimal point, and slash, respectively...
      INTEGER	ib, ie, i0, i9, iEE, iDD, ip, im, id, is

C     Flags 'lsgn', 'lexp' and 'ldp' are raised when a + or - sign, an
C     exponent character, and a decimal point are found, respectively;
C     'll' is temporary boolean...
      LOGICAL	lsgn, lexp, ldp, ll

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfgkvale): '

C --- Initialize local variables...
      CALL cinit(ctmp)
      CALL cinit(cval)
      ib  = ICHAR(' ')
      ie  = ICHAR('=')
      i0  = ICHAR('0')
      i9  = ICHAR('9')
      iEE = ICHAR('E')
      iDD = ICHAR('D')
      ip  = ICHAR('+')
      im  = ICHAR('-')
      id  = ICHAR('.')
      is  = ICHAR('/')
      lsgn = .FALSE.
      lexp = .FALSE.
      ldp  = .FALSE.
      nd  = 0

C --- Test that byte 9 in 'crec' is indeed an equals sign (captures user
C     error of calling 'rjfgkvale' before 'rjfvkval')...
      cc = crec(9:9)
      ic = ICHAR(cc)
      IF (ic.NE.ie) THEN
C        Not an equals-sign, therefore no keyword value in this record!
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'header record does not contain a keyword value'
         ierr = 2671
         GOTO 9
      ENDIF

C --- Assign the nominal value portion of the header record to 'ctmp',
C     and convert any alphabetical characters to upper case.  In v3.0 of
C     the FITS standard free-format values are allowed to start anywhere
C     after character 10 ...
      ctmp(1:71) = crec(10:80)
      CALL ucase(ctmp,ctmp)


C     The following counters store: j = actual length (position of last
C     non-blank character) of 'ctmp', i = position of first non-blank
C     character in 'ctmp', k = position of the last character belonging
C     to the keyword value...
      j = clen(ctmp)
      i = lenc(ctmp)
      k = 0


C --- Test that the first non-blank character in the value portion of
C     the record is a plus or minus sign, a digit, or a decimal point
C     (may also capture the user error of calling 'rjfgkvale' while the
C     return value of 'rjfvkval' != 6) ...
      cc = ctmp(i:i)
      ic = ICHAR(cc)
      IF (ic.NE.ip .AND. ic.NE.im .AND. ic.NE.id
     +    .AND. (ic.LT.i0 .OR. ic.GT.i9)         ) THEN
C        Unexpected first non-blank char; keyword is not real-valued!
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value is not a real'//
     +      ' (does not start with a sign, digit, or decimal point)'
         ierr = 2672
         GOTO 9
      ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C        First non-blank character is a sign character...
         IF (idbg.GT.2) WRITE(*,*) dbid//'  keyword value starts '//
     +      'with a sign'
         lsgn = .TRUE.
      ELSE IF (ic.GE.i0 .AND. ic.LE.i9) THEN
C        First non-blank character is a digit...
         IF (idbg.GT.2) WRITE(*,*) dbid//'  keyword value starts '//
     +      'with a digit'
         nd = nd + 1
      ELSE IF (ic.EQ.id) THEN
C        First non-blank character is a decimal point...
         IF (idbg.GT.2) WRITE(*,*) dbid//'  keyword value starts '//
     +      'with a decimal point'
         ldp = .TRUE.
      ENDIF


C --- Copy the non-blank characters in 'ctmp', one character at a time,
C     to temporary value string 'cval', while testing on-the-fly for any
C     illegal character that is inconsistent with an integer value.  The
C     first char in 'cval' is blank to aid in the list-directed read ...
      m = 2
      cval(m:m) = cc

C  >> Begin-of-loop over the contiguous characters in 'ctmp', starting
C     with the 2nd non-blank character, until we find a blank or slash...
      k = i
1     k = k + 1

      cc = ctmp(k:k)
      ic = ICHAR(cc)

C     Test whether this is a valid character...
      ll = (ic.EQ.ip.OR.ic.EQ.im.OR.ic.EQ.id.OR.ic.EQ.iEE.OR.ic.EQ.iDD)
      IF (.NOT.ll .AND. (ic.LT.i0 .OR. ic.GT.i9)
     +            .AND. ic.NE.ib .AND. ic.NE.is  ) THEN
C        Illegal character: neither a sign, digit, nor decimal point...
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +      'character other than sign, decimal point, digit, or'//
     +      'exponent character'
         ierr = 2673
         GOTO 9
      ELSE IF (lexp .AND. lsgn .AND. (ic.EQ.ip .OR. ic.EQ.im)) THEN
C        More than one sign in exponent: value is not a real...
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +      'more than one sign in the exponent: not a real'
         ierr = 2674
         GOTO 9
      ELSE IF (lsgn .AND. (ic.EQ.ip .OR. ic.EQ.im)) THEN
C        More than one sign in mantissa: value is not a real...
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +      'more than one sign in the mantissa: not a real'
         ierr = 2675
         GOTO 9
      ELSE IF (lexp .AND. (ic.EQ.iEE .OR. ic.EQ.iDD) ) THEN
C        More than one exponent detected: value is not a real...
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +      'more than one exponent character: not a real'
         ierr = 2676
         GOTO 9
      ELSE IF (ldp .AND. ic.EQ.id) THEN
C        More than one decimal point detected: value is not a real...
         IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +      'more than one decimal point: not a real'
         ierr = 2677
         GOTO 9
      ELSE IF (ic.EQ.iEE .OR. ic.EQ.iDD) THEN
C        Exponent sign detected; reset sign flag; change 'E' to 'D' to
C        aid in subsequent list-directed read...
         lexp = .TRUE.
         lsgn = .FALSE.
         cc = 'D'
      ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C        Detected a sign character in the exponent...
         lsgn = .TRUE.
      ELSE IF (ic.EQ.id) THEN
C        Detected a decimal point in the mantissa...
         ldp = .TRUE.
      ELSE IF (.NOT.lexp .AND. ic.GE.i0 .AND. ic.LE.i9 ) THEN
C        Detected a digit in the mantissa...
         nd = nd + 1
      ELSE IF (ic.EQ.ib .OR. ic.EQ.is) THEN
C        We have read all characters representing the real value; if
C        not in exponential notation already, append 'D0' to aid in the
C        subsequent list-directed read...
         IF (.NOT.lexp) THEN
            m = m + 2
            cval((m-1):m) = 'D0'
         ENDIF
         cc = ' '
      ENDIF

C     Character is indeed valid, so copy it to 'cval'...
      m = m + 1
      cval(m:m) = cc

C     Decide whether to continue testing the next character...
      IF (ic.NE.ib .AND. ic.NE.is .AND. k.LT.j) GOTO 1

C  << End-of-loop.  Counter i may be re-used.


C --- Test whether we exited the loop because of an embedded space...
      IF (ic.EQ.ib .AND. k.LT.j) THEN
C        Indeed, we exited due to a space; next non-blank character, if
C        any, must be a / ...
2        k = k + 1
         cc = ctmp(k:k)
         ic = ICHAR(cc)
         IF (ic.EQ.ib .AND. k.LT.j) GOTO 2

         IF (k.LT.j .AND. ic.NE.ib .AND. ic.NE.is) THEN
C           Detected a non-blank character other than a / ...
            IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword '//
     +         'value with embedded space character(s)'
            ierr = 2678
            GOTO 9
         ENDIF
      ENDIF
C     Counters i and j may be re-used.


C --- Test that the mantissa contained at least one digit...
      IF (nd.LT.1) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword '//
     +      'value without any digits in the mantissa'
         ierr = 2679
         GOTO 9
      ENDIF

C --- Test that the final character of any exponent is a digit...
      cc = cval((m-1):(m-1))
      ic = ICHAR(cc)
      IF (lexp .AND. (ic.LT.i0 .OR. ic.GT.i9)) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'malformatted keyword '//
     +      'value without any digits in the exponent'
         ierr = 2680
         GOTO 9
      ENDIF


C --- Try to read the real keyword value from string 'cval'...
      IF (idbg.GT.2) WRITE(*,*) dbid//'  cval="'//cval(1:m)//'"'
      READ(cval,*,IOSTAT=ierr) rtmp
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'failed to read REAL keyword value from record'
         ierr = 2681
         GOTO 9
      ENDIF
C     Return the keyword value (cast to default REAL data type)...
      rval = rtmp
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),E17.9e3)') ' '//dbid//
     +   '  rtmp=',rtmp,', rval=',rval


C --- Compare the number of digits in the mantissa to the estimated
C     precision required to faithfully represent the keyword value and
C     issue a warning, should we suffer significant loss of precision...
      np = 99
      rdev = DABS((rtmp-DBLE(rval))/rtmp)
      IF (rdev.GT.0.0d0) np = ABS(INT(LOG10(rdev)))
      IF (idbg.GT.2) WRITE(*,'((A),E21.13e3,(A),I2,(A),I2,(A))')
     +   ' '//dbid//'  rdev=',rdev,' (nd=',nd,',np=',np,')'
      i = clen(dbid) - 1
      j = clen(crec(1:8))
      IF (nd.GT.np) WRITE(*,*) 'WARNING'//dbid(6:i)//' value of '//
     +   crec(1:j)//' suffered loss of precision (O{10^',-(np+1),'})'


C --- Return a possibly modified version of input header record 'crec',
C     with the keyword value (and comment) in fixed format...

C     First test whether the start position of any keyword comment needs
C     to be adjusted...
      k = cidx(crec((m+10):80),'/')
      k = k + (m+9)
      IF (k.GT.32) THEN
C        Shift keyword comment to the left...
         crec(32:(32+(80-k))) = crec(k:80)
         DO i = (32+(80-k)+1),80,1
            crec(i:i) = ' '
         ENDDO
      ELSE IF (k.LT.32) THEN
C        Shift keyword comment to the right...
         crec(32:80) = crec(k:(80-(32-k)))
         DO i = k,31,1
            crec(i:i) = ' '
         ENDDO
      ENDIF
C     If we temporarily added "D0", strip it from 'cval' again...
      IF (.NOT.lexp) THEN
         m = m - 2
         cval((m+1):(m+2)) = '  '
      ENDIF
C     Change any explicit exponent character 'D' back to 'E'...
      j = cidx(cval,'D')
      IF (j.NE.0) cval(j:j) = 'E'
C     Then right-justify the keyword value itself on byte 30...
      crec(11:31) = '                     '
      crec((33-m):30) = cval(2:(m-1))


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, ierr = ',ierr

      RETURN
      END


