C ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

      SUBROUTINE rjfgkvalm (crec, irval, iival, idbg, ierr)

C @(#)Subroutine: rjfgkvalm          Author: R.A. Jansen -- Oct 13, 2009
C @(#)
C @(#)  Purpose: Read a complex integer-valued keyword value from a FITS
C @(#)    header record 'crec', already known to contain a keyword value
C @(#)    of this type,  and return its  real part  as  'irval'  and its
C @(#)    imaginary part  as  'iival' ,  both declared  as variables  of
C @(#)    the default  INTEGER  data type.  Depending on the host OS and
C @(#)    compiler, this may accommodate 16-bit (INTEGER*2) but more ge-
C @(#)    nerally 32-bit (INTEGER*4) integers.  If the keyword value was
C @(#)    written with embedded spaces, 'crec' will be modified with the
C @(#)    value written as a comma-separated pair of integers,  enclosed
C @(#)    within parentheses,  left-justified on byte 11.   If verbosity
C @(#)    level 'idbg'>0, additional messages may be printed. Should any
C @(#)    error be detected, error status  'ierr'  will be non-zero upon
C @(#)    exit.  Its value (if >2000) uniquely identifies  a known error
C @(#)    condition.
C @(#)
C @(#)  Calling sequence:
C @(#)    CALL rjfgkvalm (crec, irval, iival, idbg, ierr),  where 'crec'
C @(#)    is a character string with a declared length of 80 characters.
C @(#)    'irval' and 'iival'  are integers of the default  INTEGER data
C @(#)    type.  'idbg' and 'ierr' are both integers.
C @(#)
C @(#)  Note: this task is meant to be run after 'rjfvkval' (when 'ityp'
C @(#)    has the return value of 8) and 'rjfgkwrd'.
C @(#)
C @(#)  Dependencies:	clen, cinit, cidx, lenc, rjfvkmpq
C @(#)
C @(#)  Revision history:
C @(#)    Oct  7, 2009 [RAJ] -- Initial version of this routine.
C @(#)    Oct 12, 2009 [RAJ] -- Fixed modification of 'crec'; more error
C @(#)                          conditions captured.
C @(#)    Oct 13, 2009 [RAJ] -- Split off common code of tests & prepa-
C @(#)                          ratory tasks in routine 'rjfgkval[mpq]'
C @(#)                          to 'rjfvkmpq'.  Updated error codes.
C @(#)------------------------------------------------------------------

      IMPLICIT NONE

C --- Declaration of input/return variables...
      CHARACTER	crec*80
      INTEGER	irval, iival, idbg, ierr

C --- Declaration of local variables...
      CHARACTER ctmp*80, cval*80, sval*80, dbid*19, cc
      INTEGER*4	jval, jtmp
      INTEGER	clen, lenc, cidx, ic, i, j, k, m, n

C     The following variables are shorthand integer equivalents in the
C     ASCII collating sequence of space, digits 0 and 9, plus and minus,
C     respectively...
      INTEGER	ib, i0, i9, ip, im

C     Flag 'lsgn' is raised if a + or - sign is found; 'last' is raised
C     when a (terminating) space character is found...
      LOGICAL	lsgn, last

C     ------------------------------------------------------------------

C --- Return for non-zero input error status...
      IF (ierr.NE.0) RETURN

C --- Initialize the debugging ID string...
      dbid  = 'DEBUG(rjfgkvalm): '

C --- Initialize the output variable...
      irval = 0
      iival = 0

C --- Initialize local variables (space, =, 0, 9, +, -, / )...
      ib = ICHAR(' ')
      i0 = ICHAR('0')
      i9 = ICHAR('9')
      ip = ICHAR('+')
      im = ICHAR('-')

C --- Test whether default INTEGER accommodates 16 or 32 bit...
      jtmp = 32767
      irval = jtmp + 1
      IF (irval.LT.0) THEN
         IF (idbg.GT.2) WRITE(*,*) dbid//'  INTEGER = INTEGER*2'
         jtmp = 32767
      ELSE
         IF (idbg.GT.2) WRITE(*,*) dbid//'  INTEGER = INTEGER*4'
         jtmp = 2147483647
      ENDIF
      irval = 0

C --- Test that header record 'crec' indeed contains a complex-valued
C     keyword value and appears to have a valid format; then return as
C     character string 'ctmp' a space-separated copy of the real and
C     imaginary parts of that complex value (without the parentheses),
C     along with the byte positions in 'ctmp' of the first non-blank
C     character ('i'), the separating space ('j'; originally the comma),
C     and last non-blank character ('k') ...
      CALL rjfvkmpq (dbid,crec,ctmp,i,j,k,idbg,ierr)
      IF (ierr.NE.0) GOTO 9


C - - - - - - - -   Real part of complex keyword value   - - - - - - - -

      CALL cinit(cval)
      lsgn = .FALSE.
      last = .FALSE.

C --- Test that the first non-blank character in the real part of the
C     complex keyword value is either a sign or a digit...
      cc = ctmp(i:i)
      ic = ICHAR(cc)
      IF (ic.NE.ip .AND. ic.NE.im .AND. (ic.LT.i0 .OR. ic.GT.i9) ) THEN
C        Unexpected first non-blank char; real part not integer-valued!
         IF (idbg.GT.0) WRITE(*,*) dbid//'real part of keyword value '//
     +      'is not an integer (does not start with a sign or digit)'
         ierr = 2721
         GOTO 9
      ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C        First non-blank character is a sign character...
         IF (idbg.GT.2) WRITE(*,*) dbid//'  real part of keyword valu'//
     +      'e starts with a sign'
         lsgn = .TRUE.
      ENDIF

C --- Copy the non-blank characters of the real part of the complex key-
C     word value from 'ctmp' to temporary value string 'cval', one char
C     at a time, while testing on-the-fly for any illegal character that
C     is inconsistent with an integer value. The first char in 'cval' is
C     blank to aid in list-directed read ...
      m = 2
      cval(m:m) = cc
      DO n = (i+1),(j-1),1

         cc = ctmp(n:n)
         ic = ICHAR(cc)

C        Test whether this is a valid character...
         IF (last .AND. ic.NE.ib) THEN
C           Detected an embedded blank character in the real part...
            IF (idbg.GT.0) WRITE(*,*) dbid//'real part of keyword val'//
     +         'ue contains embedded space character(s)'
            ierr = 2722
         ELSE IF (ic.GE.i0 .AND. ic.LE.i9) THEN
C           Valid character (digit), so copy it to 'cval'...
            m = m + 1
            cval(m:m) = cc
         ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C           Spurious extra or embedded sign detected: not an integer...
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +         'a spurious extra or embedded sign: not an integer'
            ierr = 2723
         ELSE IF (ic.EQ.ib) THEN
C           We must have read all characters representing the integer...
            last = .TRUE.
         ELSE
C           Illegal character: neither a sign nor a digit...
            IF (idbg.GT.0) WRITE(*,*) dbid//'real part of keyword val'//
     +         'ue contains character other than sign or digits'
            ierr = 2724
         ENDIF
         IF (ierr.NE.0) GOTO 9

      ENDDO
      cval((m+1):(m+1)) = ' '

C --- Try to read the real integer value from string 'cval'...
      jval = 0
      READ(cval,*,IOSTAT=ierr) jval
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'failed to read complex INTEGER keyword value from record'
         ierr = 2729
         GOTO 9
      ENDIF
      IF (jval.LT.-jtmp.OR.jval.GE.jtmp) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'value read from record is '//
     +      'out of INTEGER range'
         ierr = 2730
         GOTO 9
      ENDIF
C     Return the real part of the complex value (cast to default INTEGER
C     data type)...
      irval = jval

C --- Store formatted contents of 'cval' for later use...
      CALL cinit(sval)
      sval(1:1) = '('
      IF (lsgn) THEN
         sval(2:m) = cval(2:m)
      ELSE
         sval(2:2) = '+'
         sval(3:(m+1)) = cval(2:m)
         m = m + 1
      ENDIF
      sval((m+1):(m+1)) = ','

C - - - - - - - - Imaginary part of complex keyword value  - - - - - - -

      CALL cinit(cval)
      lsgn = .FALSE.
      last = .FALSE.

C     Position of first and last non-blank char in the imaginary part...
      i = lenc(ctmp((j+1):80))
      i = i + j
      k = clen(ctmp)

C --- Test that the first non-blank character in the imaginary part of
C     the complex keyword value is either a sign or a digit...
      cc = ctmp(i:i)
      ic = ICHAR(cc)
      IF (ic.NE.ip .AND. ic.NE.im .AND. (ic.LT.i0 .OR. ic.GT.i9) ) THEN
C        Unexpected first non-blank char; imaginary part not an integer!
         IF (idbg.GT.0) WRITE(*,*) dbid//'real part of keyword value '//
     +      'is not an integer (does not start with a sign or digit)'
         ierr = 2725
         GOTO 9
      ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C        First non-blank character is a sign character...
         IF (idbg.GT.2) WRITE(*,*) dbid//'  imaginary part of keyword'//
     +      ' value starts with a sign'
         lsgn = .TRUE.
      ENDIF

C --- Copy the non-blank characters of the imaginary part of the complex
C     keyword value from 'ctmp' to temporary value string 'cval', one at
C     a time, while testing on-the-fly for any illegal character that is
C     inconsistent with an integer value.  The first char in 'cval' is
C     blank to aid in list-directed read ...
      m = 2
      cval(m:m) = cc
      DO n = (i+1),k,1

         cc = ctmp(n:n)
         ic = ICHAR(cc)

C        Test whether this is a valid character...
         IF (last .AND. ic.NE.ib) THEN
C           Detected an embedded blank character in the imaginary part..
            IF (idbg.GT.0) WRITE(*,*) dbid//'imaginary part of keywor'//
     +         'd value contains embedded space character(s)'
            ierr = 2726
         ELSE IF (ic.GE.i0 .AND. ic.LE.i9) THEN
C           Valid character (digit), so copy it to 'cval'...
            m = m + 1
            cval(m:m) = cc
         ELSE IF (ic.EQ.ip .OR. ic.EQ.im) THEN
C           Spurious extra or embedded sign detected: not an integer...
            IF (idbg.GT.0) WRITE(*,*) dbid//'keyword value contains '//
     +         'a spurious extra or embedded sign: not an integer'
            ierr = 2727
         ELSE IF (ic.EQ.ib) THEN
C           We must have read all characters representing the integer...
            last = .TRUE.
         ELSE
C           Illegal character: neither a sign nor a digit...
            IF (idbg.GT.0) WRITE(*,*) dbid//'imaginary part of keywor'//
     +         'd value contains character other than sign or digits'
            ierr = 2728
         ENDIF
         IF (ierr.NE.0) GOTO 9

      ENDDO
      cval((m+1):(m+1)) = ' '

C --- Try to read the imaginary integer value from string 'cval'...
      jval = 0
      READ(cval,*,IOSTAT=ierr) jval
      IF (ierr.NE.0) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//
     +      'failed to read complex INTEGER keyword value from record'
         ierr = 2729
         GOTO 9
      ENDIF
      IF (jval.LT.-jtmp.OR.jval.GE.jtmp) THEN
         IF (idbg.GT.0) WRITE(*,*) dbid//'value read from record is '//
     +      'out of INTEGER range'
         ierr = 2730
         GOTO 9
      ENDIF
C     Return the imaginary part of the complex value (cast to default
C     INTEGER data type)...
      iival = jval

C --- Append formatted contents of 'cval' to 'sval' for later use...
      i = clen(sval)
      j = i + 1
      k = i + m - 1
      IF (lsgn) THEN
         sval(j:k) = cval(2:m)
      ELSE
         sval(j:j) = '+'
         sval((j+1):(k+1)) = cval(2:m)
         m = m + 1
      ENDIF
      sval((i+m):(i+m)) = ')'
      j = i + m


C - - - - - - - -  Return value of header record 'crec'  - - - - - - - -

C --- Return a possibly modified version of input header record 'crec',
C     with the keyword value (and comment) in fixed format...

      i = MAX(32,(10+j+2))
      k = cidx(crec((m+10):80),'/')
      k = k + (m+9)

C     First test whether the start position of any keyword comment needs
C     to be adjusted...
      IF (k.GT.i) THEN
C        Shift keyword comment to the left...
         crec(i:(i+(80-k))) = crec(k:80)
         DO n = (i+(80-k)+1),80,1
            crec(n:n) = ' '
         ENDDO
      ELSE IF (k.LT.i) THEN
C        Shift keyword comment to the right...
         crec(i:80) = crec(k:(80-(i-k)))
         DO n = k,(i-1),1
            crec(n:n) = ' '
         ENDDO
      ENDIF
C     Then left-justify the keyword value itself on byte 11 ...
      DO n = 11,(i-1),1
         crec(n:n) = ' '
      ENDDO
      crec(11:(10+j)) = sval(1:j)


9     IF (idbg.GT.2) WRITE(*,*) dbid//crec(1:clen(crec))
      IF (idbg.GT.1) WRITE(*,*) dbid//'on exit, irval=',irval,
     +   ', iival=',iival,', ierr=',ierr

      RETURN
      END


